#include <iostream>
using namespace std;

class Point {
private:
  double x;
  double y;
public:
  Point() : x(0.0),y(0.0) {}
  Point(
    double x_,
    double y_
  ) :
    x(x_),y(y_)
  {}
  Point(const Point& P) : x(P.x),y(P.y) {}
  ~Point() {}
  Point& operator=(const Point& P) {
    x=P.x;
    y=P.y;
    return(*this);
  }
  friend ostream& operator<<(
    ostream& out,
    const Point& P
  ) {
    out << "(" << P.x << "," << P.y << ")\n";
  }
  void Write(void) const {
     cout << "(" << x << "," << y << ")\n";
  }
  void Translate(
    double x_,
    double y_
  ) {
    x+=x_;
    y+=y_;
  }
};

class Shape {
public:
  enum Color { red, green, blue };
private:
  Color color;
public:
  Shape() : color(red) {}
  Shape(
    const Shape& S
  ) :
    color(S.color)
  {}
  virtual ~Shape() {
    cout << "destroying Shape\n";
  }
  // tratar tambien el caso: f = f;
  Shape& operator=(
    const Shape& S
  ) {
    if(this!=&S) {
      color=S.color;
    }
    return *this;
  }
  void WriteColor(void) const {
    switch(color) {
      case red: cout << "red\n"; break;
      case green: cout << "green\n"; break;
      case blue: cout << "blue\n"; break;
    }
  };
  void WriteColor(
    const Color& color
  ) const {
    // el color de la forma seria:
    //  Shape::color
    switch(color) {
      case red: cout << "red\n"; break;
      case green: cout << "green\n"; break;
      case blue: cout << "blue\n"; break;
    }
  };
  virtual void Write(void) const = 0;
  virtual void Translate(
    double x,
    double y) = 0;
};

class Triangle : public Shape {
private:
  Point A;
  Point B;
  Point C;
public:
  Triangle() : Shape(),A(),B(),C() {}
  Triangle(
    const Point& A_,
    const Point& B_,
    const Point& C_
  ) :
    Shape(),A(A_),B(B_),C(C_) {}
  Triangle(
    const Triangle& T
  ) :
    Shape(T),
    A(T.A),
    B(T.B),
    C(T.C)
  {}
  virtual ~Triangle(
  ) {
    cout << "destroying Triangle\n";
  }
  virtual void Write(void) const {
#ifdef NEVER
    cout << "A: "; A.Write();
    cout << "B: "; B.Write();
    cout << "C: "; C.Write();
#endif
    cout << "A: " << A << endl;
    cout << "B: " << B << endl;
    cout << "C: " << C << endl;
    WriteColor();
  }
  virtual void Translate(
    double x,
    double y
  ) {
    A.Translate(x,y);
    B.Translate(x,y);
    C.Translate(x,y);
  }
};

class TriangleFilled : public Triangle {
private:
  Color fill_color;
public:
  TriangleFilled()
    : Triangle(),fill_color(red) {}
  TriangleFilled(
    const Triangle& T,
    const Color& fill_color_
  ) :
    Triangle(T),fill_color(fill_color_) {}
  TriangleFilled(
    const TriangleFilled& T
  ) :
    Triangle(T),
    fill_color(T.fill_color)
  {}
  virtual ~TriangleFilled(
  ) {
    cout << "destroying TriangleFilled\n";
  }
  virtual void Write(void) const {
    Triangle::Write();
    WriteColor(fill_color);
  }
};

class Circle : public Shape {
private:
  Point center;
  double radius;
public:
  Circle() : Shape(),center(),radius(0.0) {}
  Circle(
    const Point& center_,
    double radius_
  ) :
    Shape(),center(center_),radius(radius_)
  {}
  Circle(
    const Circle& C
  ) :
    Shape(C),
    center(C.center),
    radius(C.radius)
  {}
  virtual ~Circle(
  ) {
    cout << "destroying Circle\n";
  }
  virtual void Write(void) const {
    cout << "center: " << center << endl;
    cout << "radius: " << radius << endl;
    WriteColor();
  }
  virtual void Translate(
    double x,
    double y
  ) {
    center.Translate(x,y);
  }
};

int main(
) {
  Shape* shape=0;
  cout
    << "0 for triangle\n"
    << "1 for triangle filled\n"
    << "2 for circle\n";
  int input;
  cin >> input;

  switch(input) {
    case 0:
      shape=new Triangle(Point(1,1),Point(2,2),Point(2,0));
      break;
    case 1: {
        // estas llaves son necesarias para que podamos definir
        // una variable local dentro de este bloque del case
      Triangle T(
        Point(1,1),Point(2,2),Point(2,0)
      );
      shape=new TriangleFilled(T,Shape::green);
      break;
    }
    default:
      shape=new Circle(Point(4,4),4);
      break;
  }

  shape->Write();
  cout << endl;

  shape->Translate(5,3);
  shape->Write();
  cout << endl;

  delete shape;
  return 0;
}

