#include <iostream>
using namespace std;

class Point {
private:
  double x;
  double y;
public:
  Point() : x(0.0),y(0.0) {}
  Point(
    double x_,
    double y_
  ) :
    x(x_),y(y_)
  {}
  Point(const Point& P) : x(P.x),y(P.y) {}
  ~Point() {}
  Point& operator=(const Point& P) {
    x=P.x;
    y=P.y;
    return(*this);
  }
  void Write(void) const {
    cout << "(" << x << "," << y << ")\n";
  }
  void Translate(
    double x_,
    double y_
  ) {
    x+=x_;
    y+=y_;
  }
};

class Shape {
public:
  enum Color { red, green, blue };
private:
  Color color;
public:
  Shape() : color(red) {}
  Shape(
    const Shape& S
  ) :
    color(S.color)
  {}
  ~Shape() {
    cout << "destroying Shape\n";
  }
  // f = f;
  Shape& operator=(
    const Shape& S
  ) {
    if(this!=&S) {
      color=S.color;
    }
    return *this;
  }
  void WriteColor(void) const {
    switch(color) {
      case red: cout << "red\n"; break;
      case green: cout << "green\n"; break;
      case blue: cout << "blue\n"; break;
    }
  };
  virtual void Write(void) const = 0;
#ifdef NEVER
  virtual void TellMe(void) {
    cout << "it's a shape\n";
  }
  virtual void Translate(
    double x,
    double y
  ) = 0;
#endif
};

typedef Shape* pShape;

class Triangle : public Shape {
private:
  Point A;
  Point B;
  Point C;
public:
  Triangle() : Shape(),A(),B(),C() {}
  Triangle(
    const Point& A_,
    const Point& B_,
    const Point& C_
  ) :
    Shape(),A(A_),B(B_),C(C_) {}
  Triangle(
    const Triangle& T
  ) :
    Shape(T),
    A(T.A),
    B(T.B),
    C(T.C)
  {}
  virtual ~Triangle(
  ) {
    cout << "destroying Triangle\n";
  }
  virtual void Write(void) const {
    cout << "A: "; A.Write();
    cout << "B: "; B.Write();
    cout << "C: "; C.Write();
    WriteColor();
  }
//  void TellMe(void) {
//    cout << "it's a triangle\n";
//  }
  virtual void Translate(
    double x,
    double y
  ) {
    A.Translate(x,y);
    B.Translate(x,y);
    C.Translate(x,y);
  }
};

class Circle : public Shape {
private:
  Point center;
  double radius;
public:
  Circle() : Shape(),center(),radius(0.0) {}
  Circle(
    const Point& center_,
    double radius_
  ) :
    Shape(),center(center_),radius(radius_)
  {}
  Circle(
    const Circle& C
  ) :
    Shape(C),
    center(C.center),
    radius(C.radius)
  {}
  virtual ~Circle(
  ) {
    cout << "destroying Circle\n";
  }
  virtual void Write(void) const {
    cout << "center: "; center.Write();
    cout << "radius: " << radius << endl;
    WriteColor();
  }
  void TellMe(void) {
    cout << "it's a circle\n";
  }
  virtual void Translate(
    double x,
    double y
  ) {
    center.Translate(x,y);
  }
};

int main(
) {
  Shape S;
#ifdef NEVER
  Triangle T0;
  T0.Write();
  Triangle T1(
    Point(1,2),Point(2,3),Point(4,3)
  );
  T1.Write();
  
  Circle C(Point(5,5),7.0);
  C.Write();
#endif

  cout
    << "0 for triangle\n"
    << "1 for circle\n";
  int input;
  cin >> input;

  // pShape shape;
  Shape* shape;
  switch(input) {
    case 0:
      shape=new Triangle(
        Point(1,1),Point(1,1),Point(1,1)
      );
      break;
    default:
      shape=new Circle(
        Point(4,4),4
      );
      break;
  }

  shape->Write();
  cout << endl;
#ifdef NEVER
  shape->TellMe();

  shape->Translate(5,5);
  shape->Write();
#endif

  delete shape;
  return 0;
}

